import numpy as np
import xarray as xr

def interrupt(*arg):
    # pauses execution outputting values referenced by *arg 
    # for debugging purpose only
    print('----\ninterrupt in')

    for a in arg:
        print(a)
    print('--------------')
    inp = input('any key')


def round_sig(x, name, sig=15):
    # gives x with sig significant figures, works with xarray.DataArray
    if type(x) == xr.DataArray:
        log10_vec = np.vectorize(pyfunc=lambda x: 0 if x==0 else np.log10(abs(x)) )
        y = xr.zeros_like(x)
        y.data = log10_vec(x)
        return np.round(x, sig-int(np.amin(np.floor(y)))-1)
    else:
        try:
            return 0 if x==0 else np.round(x, sig-int(np.amin(np.floor(np.log10(abs(x)))))-1)
        except:
            print('----\nError raised in round_sig with name/type of argument: ', name, '\n', type(x), '\n----')
            raise 
def single_precision(x):
    # gives single (32 bit float) representation as done in numpy.float32
    # notice that it may differ from other double->single conversions (e.g. this happens in Vensim)
    try:
        return x.astype('float32', copy=False)
    except:
        return np.float32(x)

def single_precision_v2(x):
    # vensim-like double->single converter
    # to be implemented later on
    return single_precision(x)

def less_thr_func(x, y):
    # gives the value that exceeds the single precision rounding error of x
    smallest_value = 1e-7
    ret = (abs(x) + abs(y))*1e-7
    try:
        ret = ret.where(ret > 1e-14, smallest_value)
    except:
        ret = np.where(ret > 1e-14, ret, smallest_value)
    return ret


def test_nans(ret, name):
    #print(ret)
    #inp = input(f'Entered {name}')
    #print(f'Entered {name}')
    #with open('log.txt', 'a') as f:
    #    f.write(str(name) + ',')
    try:
        ret.name = name 
        try:
            if (True in np.isnan(ret).values):
                interrupt('NaN occured at:', True in np.isnan(ret).values, ret.name, ret)
        except Exception as e:
            print(e)
            inp = input(f'ret, name = {ret}, {name}')
    except Exception as e:
        #print('Error:', e)
        #print(f'in name: {name}')
        #print(f'type = {type(ret)}')
        if type(ret) == xr.DataArray:
            print('ret = ', ret)
            raise e
        #print(type(ret))
        
        if ret is None:
            interrupt('None in: ', ret, name)
        try:
            if np.any(np.isnan(ret)):
                interrupt('NaN occured at:', ret, name)    
        except:
            print(e)
            interrupt('Check for np.isnan failed for ', ret, name)